@@ -230,20 +230,36 @@ class HTMLTestRunner(object):
         Override this to add custom attributes.
         """
         start_time_format = str(self.start_time)[:19]
-        duration = str(self.end_time - self.start_time)
+        end_time_format = str(self.end_time)[:19]
+        duration = str(self.end_time - self.start_time)[:-3]
 
+        RunResult.start_time = start_time_format
+        RunResult.end_time = end_time_format
+        RunResult.duration = duration
         RunResult.passed = result.success_count
         RunResult.failed = result.failure_count
         RunResult.errors = result.error_count
         RunResult.skipped = result.skip_count
         count = RunResult.passed + RunResult.failed + RunResult.errors + RunResult.skipped
-        p_percent = '{:.2%}'.format(RunResult.passed / count)
-        e_percent = '{:.2%}'.format(RunResult.errors / count)
-        f_percent = '{:.2%}'.format(RunResult.failed / count)
-        s_percent = '{:.2%}'.format(RunResult.skipped / count)
+        p_percent = '0.00'
+        e_percent = '0.00'
+        f_percent = '0.00'
+        s_percent = '0.00'
+        if count > 0:
+            p_percent = '{:.2%}'.format(RunResult.passed / count)
+            e_percent = '{:.2%}'.format(RunResult.errors / count)
+            f_percent = '{:.2%}'.format(RunResult.failed / count)
+            s_percent = '{:.2%}'.format(RunResult.skipped / count)
+
+        RunResult.count = count
+        RunResult.pass_rate = p_percent
+        RunResult.error_rate = e_percent
+        RunResult.failure_rate = f_percent
+        RunResult.skip_rate = s_percent
 
         base_info = {
             "start_time": start_time_format,
+            "end_time": end_time_format,
             "duration": duration
         }
 
@@ -297,6 +313,7 @@ class HTMLTestRunner(object):
         heading = env.get_template(heading_html).render(
             title=self.title,
             start_time=base["start_time"],
+            end_time=base["end_time"],
             duration=base["duration"],
             tester=self.tester,
             description=self.description,
@@ -321,17 +338,9 @@ class HTMLTestRunner(object):
                 if num == 0:
                     num_pass += 1
                 elif num == 1:
-                    if self.test_obj != test:
-                        self.test_obj = test
-                        num_fail += 1
-                    else:
-                        num_fail += 0
+                    num_fail += 1
                 elif num == 2:
-                    if self.test_obj != test:
-                        self.test_obj = test
-                        num_error += 1
-                    else:
-                        num_error += 0
+                    num_fail += 1
                 else:
                     num_skip += 1
 
@@ -447,8 +456,8 @@ class HTMLTestRunner(object):
     def send_email(
             user: str,
             password: str,
-            host: str, to: any,
-            subject=None,
+            host: str,
+            to: any,
             attachments=None):
         """
         Send test result to email
@@ -456,13 +465,10 @@ class HTMLTestRunner(object):
         :param password:
         :param host:
         :param to:
-        :param subject:
         :param attachments:
         """
-        if subject is None:
-            subject = "XTestRunner Test Results"
         smtp = SMTP(user=user, password=password, host=host)
-        smtp.sender(to=to, subject=subject, attachments=attachments)
+        smtp.sender(to=to, attachments=attachments)
 
     @staticmethod
     def send_dingtalk(
